"""Feature map for the kaggle competition 'santander customer satisfaction."""

from collections import OrderedDict

from ...data_types import FeatureType


def get_feature_map():
    return OrderedDict([
        ("delta_imp_amort_var18_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_amort_var34_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_aport_var13_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_aport_var17_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_aport_var33_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_compra_var44_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_reemb_var13_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_reemb_var17_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_reemb_var33_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_trasp_var17_in_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_trasp_var17_out_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_trasp_var33_in_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_trasp_var33_out_1y3", FeatureType.CONTINUOUS),
        ("delta_imp_venta_var44_1y3", FeatureType.CONTINUOUS),
        ("delta_num_aport_var13_1y3", FeatureType.CONTINUOUS),
        ("delta_num_aport_var17_1y3", FeatureType.CONTINUOUS),
        ("delta_num_aport_var33_1y3", FeatureType.CONTINUOUS),
        ("delta_num_compra_var44_1y3", FeatureType.CONTINUOUS),
        ("delta_num_reemb_var13_1y3", FeatureType.CONTINUOUS),
        ("delta_num_reemb_var17_1y3", FeatureType.CONTINUOUS),
        ("delta_num_reemb_var33_1y3", FeatureType.CONTINUOUS),
        ("delta_num_trasp_var17_in_1y3", FeatureType.CONTINUOUS),
        ("delta_num_trasp_var17_out_1y3", FeatureType.CONTINUOUS),
        ("delta_num_trasp_var33_in_1y3", FeatureType.CONTINUOUS),
        ("delta_num_trasp_var33_out_1y3", FeatureType.CONTINUOUS),
        ("delta_num_venta_var44_1y3", FeatureType.CONTINUOUS),
        ("imp_amort_var18_hace3", FeatureType.CONTINUOUS),
        ("imp_amort_var18_ult1", FeatureType.CONTINUOUS),
        ("imp_amort_var34_hace3", FeatureType.CONTINUOUS),
        ("imp_amort_var34_ult1", FeatureType.CONTINUOUS),
        ("imp_aport_var13_hace3", FeatureType.CONTINUOUS),
        ("imp_aport_var13_ult1", FeatureType.CONTINUOUS),
        ("imp_aport_var17_hace3", FeatureType.CONTINUOUS),
        ("imp_aport_var17_ult1", FeatureType.CONTINUOUS),
        ("imp_aport_var33_hace3", FeatureType.CONTINUOUS),
        ("imp_aport_var33_ult1", FeatureType.CONTINUOUS),
        ("imp_compra_var44_hace3", FeatureType.CONTINUOUS),
        ("imp_compra_var44_ult1", FeatureType.CONTINUOUS),
        ("imp_ent_var16_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var39_comer_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var39_comer_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var39_efect_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var39_efect_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var39_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var40_comer_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var40_comer_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var40_efect_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var40_efect_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var40_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var41_comer_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var41_comer_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var41_efect_ult1", FeatureType.CONTINUOUS),
        ("imp_op_var41_efect_ult3", FeatureType.CONTINUOUS),
        ("imp_op_var41_ult1", FeatureType.CONTINUOUS),
        ("imp_reemb_var13_hace3", FeatureType.CONTINUOUS),
        ("imp_reemb_var13_ult1", FeatureType.CONTINUOUS),
        ("imp_reemb_var17_hace3", FeatureType.CONTINUOUS),
        ("imp_reemb_var17_ult1", FeatureType.CONTINUOUS),
        ("imp_reemb_var33_hace3", FeatureType.CONTINUOUS),
        ("imp_reemb_var33_ult1", FeatureType.CONTINUOUS),
        ("imp_sal_var16_ult1", FeatureType.CONTINUOUS),
        ("imp_trans_var37_ult1", FeatureType.CONTINUOUS),
        ("imp_trasp_var17_in_hace3", FeatureType.CONTINUOUS),
        ("imp_trasp_var17_in_ult1", FeatureType.CONTINUOUS),
        ("imp_trasp_var17_out_hace3", FeatureType.CONTINUOUS),
        ("imp_trasp_var17_out_ult1", FeatureType.CONTINUOUS),
        ("imp_trasp_var33_in_hace3", FeatureType.CONTINUOUS),
        ("imp_trasp_var33_in_ult1", FeatureType.CONTINUOUS),
        ("imp_trasp_var33_out_hace3", FeatureType.CONTINUOUS),
        ("imp_trasp_var33_out_ult1", FeatureType.CONTINUOUS),
        ("imp_var43_emit_ult1", FeatureType.CONTINUOUS),
        ("imp_var7_emit_ult1", FeatureType.CONTINUOUS),
        ("imp_var7_recib_ult1", FeatureType.CONTINUOUS),
        ("imp_venta_var44_hace3", FeatureType.CONTINUOUS),
        ("imp_venta_var44_ult1", FeatureType.CONTINUOUS),
        ("ind_var1", FeatureType.CONTINUOUS),
        ("ind_var10_ult1", FeatureType.CONTINUOUS),
        ("ind_var10cte_ult1", FeatureType.CONTINUOUS),
        ("ind_var12", FeatureType.CONTINUOUS),
        ("ind_var12_0", FeatureType.CONTINUOUS),
        ("ind_var13", FeatureType.CONTINUOUS),
        ("ind_var13_0", FeatureType.CONTINUOUS),
        ("ind_var13_corto", FeatureType.CONTINUOUS),
        ("ind_var13_corto_0", FeatureType.CONTINUOUS),
        ("ind_var13_largo", FeatureType.CONTINUOUS),
        ("ind_var13_largo_0", FeatureType.CONTINUOUS),
        ("ind_var13_medio", FeatureType.CONTINUOUS),
        ("ind_var13_medio_0", FeatureType.CONTINUOUS),
        ("ind_var14", FeatureType.CONTINUOUS),
        ("ind_var14_0", FeatureType.CONTINUOUS),
        ("ind_var17", FeatureType.CONTINUOUS),
        ("ind_var17_0", FeatureType.CONTINUOUS),
        ("ind_var18", FeatureType.CONTINUOUS),
        ("ind_var18_0", FeatureType.CONTINUOUS),
        ("ind_var19", FeatureType.CONTINUOUS),
        ("ind_var1_0", FeatureType.CONTINUOUS),
        ("ind_var2", FeatureType.CONTINUOUS),
        ("ind_var20", FeatureType.CONTINUOUS),
        ("ind_var20_0", FeatureType.CONTINUOUS),
        ("ind_var24", FeatureType.CONTINUOUS),
        ("ind_var24_0", FeatureType.CONTINUOUS),
        ("ind_var25", FeatureType.CONTINUOUS),
        ("ind_var25_0", FeatureType.CONTINUOUS),
        ("ind_var25_cte", FeatureType.CONTINUOUS),
        ("ind_var26", FeatureType.CONTINUOUS),
        ("ind_var26_0", FeatureType.CONTINUOUS),
        ("ind_var26_cte", FeatureType.CONTINUOUS),
        ("ind_var27", FeatureType.CONTINUOUS),
        ("ind_var27_0", FeatureType.CONTINUOUS),
        ("ind_var28", FeatureType.CONTINUOUS),
        ("ind_var28_0", FeatureType.CONTINUOUS),
        ("ind_var29", FeatureType.CONTINUOUS),
        ("ind_var29_0", FeatureType.CONTINUOUS),
        ("ind_var2_0", FeatureType.CONTINUOUS),
        ("ind_var30", FeatureType.CONTINUOUS),
        ("ind_var30_0", FeatureType.CONTINUOUS),
        ("ind_var31", FeatureType.CONTINUOUS),
        ("ind_var31_0", FeatureType.CONTINUOUS),
        ("ind_var32", FeatureType.CONTINUOUS),
        ("ind_var32_0", FeatureType.CONTINUOUS),
        ("ind_var32_cte", FeatureType.CONTINUOUS),
        ("ind_var33", FeatureType.CONTINUOUS),
        ("ind_var33_0", FeatureType.CONTINUOUS),
        ("ind_var34", FeatureType.CONTINUOUS),
        ("ind_var34_0", FeatureType.CONTINUOUS),
        ("ind_var37", FeatureType.CONTINUOUS),
        ("ind_var37_0", FeatureType.CONTINUOUS),
        ("ind_var37_cte", FeatureType.CONTINUOUS),
        ("ind_var39", FeatureType.CONTINUOUS),
        ("ind_var39_0", FeatureType.CONTINUOUS),
        ("ind_var40", FeatureType.CONTINUOUS),
        ("ind_var40_0", FeatureType.CONTINUOUS),
        ("ind_var41", FeatureType.CONTINUOUS),
        ("ind_var41_0", FeatureType.CONTINUOUS),
        ("ind_var43_emit_ult1", FeatureType.CONTINUOUS),
        ("ind_var43_recib_ult1", FeatureType.CONTINUOUS),
        ("ind_var44", FeatureType.CONTINUOUS),
        ("ind_var44_0", FeatureType.CONTINUOUS),
        ("ind_var46", FeatureType.CONTINUOUS),
        ("ind_var46_0", FeatureType.CONTINUOUS),
        ("ind_var5", FeatureType.CONTINUOUS),
        ("ind_var5_0", FeatureType.CONTINUOUS),
        ("ind_var6", FeatureType.CONTINUOUS),
        ("ind_var6_0", FeatureType.CONTINUOUS),
        ("ind_var7_emit_ult1", FeatureType.CONTINUOUS),
        ("ind_var7_recib_ult1", FeatureType.CONTINUOUS),
        ("ind_var8", FeatureType.CONTINUOUS),
        ("ind_var8_0", FeatureType.CONTINUOUS),
        ("ind_var9_cte_ult1", FeatureType.CONTINUOUS),
        ("ind_var9_ult1", FeatureType.CONTINUOUS),
        ("num_aport_var13_hace3", FeatureType.CONTINUOUS),
        ("num_aport_var13_ult1", FeatureType.CONTINUOUS),
        ("num_aport_var17_hace3", FeatureType.CONTINUOUS),
        ("num_aport_var17_ult1", FeatureType.CONTINUOUS),
        ("num_aport_var33_hace3", FeatureType.CONTINUOUS),
        ("num_aport_var33_ult1", FeatureType.CONTINUOUS),
        ("num_compra_var44_hace3", FeatureType.CONTINUOUS),
        ("num_compra_var44_ult1", FeatureType.CONTINUOUS),
        ("num_ent_var16_ult1", FeatureType.CONTINUOUS),
        ("num_med_var22_ult3", FeatureType.CONTINUOUS),
        ("num_med_var45_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var12_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var13_corto_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var13_largo_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var13_medio_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var17_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var29_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var33_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var39_vig_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var44_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var5_ult3", FeatureType.CONTINUOUS),
        ("num_meses_var8_ult3", FeatureType.CONTINUOUS),
        ("num_op_var39_comer_ult1", FeatureType.CONTINUOUS),
        ("num_op_var39_comer_ult3", FeatureType.CONTINUOUS),
        ("num_op_var39_efect_ult1", FeatureType.CONTINUOUS),
        ("num_op_var39_efect_ult3", FeatureType.CONTINUOUS),
        ("num_op_var39_hace2", FeatureType.CONTINUOUS),
        ("num_op_var39_hace3", FeatureType.CONTINUOUS),
        ("num_op_var39_ult1", FeatureType.CONTINUOUS),
        ("num_op_var39_ult3", FeatureType.CONTINUOUS),
        ("num_op_var40_comer_ult1", FeatureType.CONTINUOUS),
        ("num_op_var40_comer_ult3", FeatureType.CONTINUOUS),
        ("num_op_var40_efect_ult1", FeatureType.CONTINUOUS),
        ("num_op_var40_efect_ult3", FeatureType.CONTINUOUS),
        ("num_op_var40_hace2", FeatureType.CONTINUOUS),
        ("num_op_var40_hace3", FeatureType.CONTINUOUS),
        ("num_op_var40_ult1", FeatureType.CONTINUOUS),
        ("num_op_var40_ult3", FeatureType.CONTINUOUS),
        ("num_op_var41_comer_ult1", FeatureType.CONTINUOUS),
        ("num_op_var41_comer_ult3", FeatureType.CONTINUOUS),
        ("num_op_var41_efect_ult1", FeatureType.CONTINUOUS),
        ("num_op_var41_efect_ult3", FeatureType.CONTINUOUS),
        ("num_op_var41_hace2", FeatureType.CONTINUOUS),
        ("num_op_var41_hace3", FeatureType.CONTINUOUS),
        ("num_op_var41_ult1", FeatureType.CONTINUOUS),
        ("num_op_var41_ult3", FeatureType.CONTINUOUS),
        ("num_reemb_var13_hace3", FeatureType.CONTINUOUS),
        ("num_reemb_var13_ult1", FeatureType.CONTINUOUS),
        ("num_reemb_var17_hace3", FeatureType.CONTINUOUS),
        ("num_reemb_var17_ult1", FeatureType.CONTINUOUS),
        ("num_reemb_var33_hace3", FeatureType.CONTINUOUS),
        ("num_reemb_var33_ult1", FeatureType.CONTINUOUS),
        ("num_sal_var16_ult1", FeatureType.CONTINUOUS),
        ("num_trasp_var11_ult1", FeatureType.CONTINUOUS),
        ("num_trasp_var17_in_hace3", FeatureType.CONTINUOUS),
        ("num_trasp_var17_in_ult1", FeatureType.CONTINUOUS),
        ("num_trasp_var17_out_hace3", FeatureType.CONTINUOUS),
        ("num_trasp_var17_out_ult1", FeatureType.CONTINUOUS),
        ("num_trasp_var33_in_hace3", FeatureType.CONTINUOUS),
        ("num_trasp_var33_in_ult1", FeatureType.CONTINUOUS),
        ("num_trasp_var33_out_hace3", FeatureType.CONTINUOUS),
        ("num_trasp_var33_out_ult1", FeatureType.CONTINUOUS),
        ("num_var1", FeatureType.CONTINUOUS),
        ("num_var12", FeatureType.CONTINUOUS),
        ("num_var12_0", FeatureType.CONTINUOUS),
        ("num_var13", FeatureType.CONTINUOUS),
        ("num_var13_0", FeatureType.CONTINUOUS),
        ("num_var13_corto", FeatureType.CONTINUOUS),
        ("num_var13_corto_0", FeatureType.CONTINUOUS),
        ("num_var13_largo", FeatureType.CONTINUOUS),
        ("num_var13_largo_0", FeatureType.CONTINUOUS),
        ("num_var13_medio", FeatureType.CONTINUOUS),
        ("num_var13_medio_0", FeatureType.CONTINUOUS),
        ("num_var14", FeatureType.CONTINUOUS),
        ("num_var14_0", FeatureType.CONTINUOUS),
        ("num_var17", FeatureType.CONTINUOUS),
        ("num_var17_0", FeatureType.CONTINUOUS),
        ("num_var18", FeatureType.CONTINUOUS),
        ("num_var18_0", FeatureType.CONTINUOUS),
        ("num_var1_0", FeatureType.CONTINUOUS),
        ("num_var20", FeatureType.CONTINUOUS),
        ("num_var20_0", FeatureType.CONTINUOUS),
        ("num_var22_hace2", FeatureType.CONTINUOUS),
        ("num_var22_hace3", FeatureType.CONTINUOUS),
        ("num_var22_ult1", FeatureType.CONTINUOUS),
        ("num_var22_ult3", FeatureType.CONTINUOUS),
        ("num_var24", FeatureType.CONTINUOUS),
        ("num_var24_0", FeatureType.CONTINUOUS),
        ("num_var25", FeatureType.CONTINUOUS),
        ("num_var25_0", FeatureType.CONTINUOUS),
        ("num_var26", FeatureType.CONTINUOUS),
        ("num_var26_0", FeatureType.CONTINUOUS),
        ("num_var27", FeatureType.CONTINUOUS),
        ("num_var27_0", FeatureType.CONTINUOUS),
        ("num_var28", FeatureType.CONTINUOUS),
        ("num_var28_0", FeatureType.CONTINUOUS),
        ("num_var29", FeatureType.CONTINUOUS),
        ("num_var29_0", FeatureType.CONTINUOUS),
        ("num_var2_0_ult1", FeatureType.CONTINUOUS),
        ("num_var2_ult1", FeatureType.CONTINUOUS),
        ("num_var30", FeatureType.CONTINUOUS),
        ("num_var30_0", FeatureType.CONTINUOUS),
        ("num_var31", FeatureType.CONTINUOUS),
        ("num_var31_0", FeatureType.CONTINUOUS),
        ("num_var32", FeatureType.CONTINUOUS),
        ("num_var32_0", FeatureType.CONTINUOUS),
        ("num_var33", FeatureType.CONTINUOUS),
        ("num_var33_0", FeatureType.CONTINUOUS),
        ("num_var34", FeatureType.CONTINUOUS),
        ("num_var34_0", FeatureType.CONTINUOUS),
        ("num_var35", FeatureType.CONTINUOUS),
        ("num_var37", FeatureType.CONTINUOUS),
        ("num_var37_0", FeatureType.CONTINUOUS),
        ("num_var37_med_ult2", FeatureType.CONTINUOUS),
        ("num_var39", FeatureType.CONTINUOUS),
        ("num_var39_0", FeatureType.CONTINUOUS),
        ("num_var4", FeatureType.CONTINUOUS),
        ("num_var40", FeatureType.CONTINUOUS),
        ("num_var40_0", FeatureType.CONTINUOUS),
        ("num_var41", FeatureType.CONTINUOUS),
        ("num_var41_0", FeatureType.CONTINUOUS),
        ("num_var42", FeatureType.CONTINUOUS),
        ("num_var42_0", FeatureType.CONTINUOUS),
        ("num_var43_emit_ult1", FeatureType.CONTINUOUS),
        ("num_var43_recib_ult1", FeatureType.CONTINUOUS),
        ("num_var44", FeatureType.CONTINUOUS),
        ("num_var44_0", FeatureType.CONTINUOUS),
        ("num_var45_hace2", FeatureType.CONTINUOUS),
        ("num_var45_hace3", FeatureType.CONTINUOUS),
        ("num_var45_ult1", FeatureType.CONTINUOUS),
        ("num_var45_ult3", FeatureType.CONTINUOUS),
        ("num_var46", FeatureType.CONTINUOUS),
        ("num_var46_0", FeatureType.CONTINUOUS),
        ("num_var5", FeatureType.CONTINUOUS),
        ("num_var5_0", FeatureType.CONTINUOUS),
        ("num_var6", FeatureType.CONTINUOUS),
        ("num_var6_0", FeatureType.CONTINUOUS),
        ("num_var7_emit_ult1", FeatureType.CONTINUOUS),
        ("num_var7_recib_ult1", FeatureType.CONTINUOUS),
        ("num_var8", FeatureType.CONTINUOUS),
        ("num_var8_0", FeatureType.CONTINUOUS),
        ("num_venta_var44_hace3", FeatureType.CONTINUOUS),
        ("num_venta_var44_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var12_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var12_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var12_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var12_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_corto_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_corto_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_corto_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_corto_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_largo_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_largo_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_largo_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_largo_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_medio_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_medio_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_medio_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var13_medio_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var17_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var17_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var17_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var17_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var29_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var29_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var29_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var29_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var33_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var33_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var33_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var33_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var44_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var44_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var44_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var44_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var5_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var5_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var5_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var5_ult3", FeatureType.CONTINUOUS),
        ("saldo_medio_var8_hace2", FeatureType.CONTINUOUS),
        ("saldo_medio_var8_hace3", FeatureType.CONTINUOUS),
        ("saldo_medio_var8_ult1", FeatureType.CONTINUOUS),
        ("saldo_medio_var8_ult3", FeatureType.CONTINUOUS),
        ("saldo_var1", FeatureType.CONTINUOUS),
        ("saldo_var12", FeatureType.CONTINUOUS),
        ("saldo_var13", FeatureType.CONTINUOUS),
        ("saldo_var13_corto", FeatureType.CONTINUOUS),
        ("saldo_var13_largo", FeatureType.CONTINUOUS),
        ("saldo_var13_medio", FeatureType.CONTINUOUS),
        ("saldo_var14", FeatureType.CONTINUOUS),
        ("saldo_var17", FeatureType.CONTINUOUS),
        ("saldo_var18", FeatureType.CONTINUOUS),
        ("saldo_var20", FeatureType.CONTINUOUS),
        ("saldo_var24", FeatureType.CONTINUOUS),
        ("saldo_var25", FeatureType.CONTINUOUS),
        ("saldo_var26", FeatureType.CONTINUOUS),
        ("saldo_var27", FeatureType.CONTINUOUS),
        ("saldo_var28", FeatureType.CONTINUOUS),
        ("saldo_var29", FeatureType.CONTINUOUS),
        ("saldo_var2_ult1", FeatureType.CONTINUOUS),
        ("saldo_var30", FeatureType.CONTINUOUS),
        ("saldo_var31", FeatureType.CONTINUOUS),
        ("saldo_var32", FeatureType.CONTINUOUS),
        ("saldo_var33", FeatureType.CONTINUOUS),
        ("saldo_var34", FeatureType.CONTINUOUS),
        ("saldo_var37", FeatureType.CONTINUOUS),
        ("saldo_var40", FeatureType.CONTINUOUS),
        ("saldo_var41", FeatureType.CONTINUOUS),
        ("saldo_var42", FeatureType.CONTINUOUS),
        ("saldo_var44", FeatureType.CONTINUOUS),
        ("saldo_var46", FeatureType.CONTINUOUS),
        ("saldo_var5", FeatureType.CONTINUOUS),
        ("saldo_var6", FeatureType.CONTINUOUS),
        ("saldo_var8", FeatureType.CONTINUOUS),
        ("var15", FeatureType.CONTINUOUS),
        ("var21", FeatureType.CONTINUOUS),
        ("var3", FeatureType.CONTINUOUS),
        ("var36", FeatureType.CONTINUOUS),
        ("var3", FeatureType.CONTINUOUS),
    ])
