"""Feature map for the kaggle competition 'homesite quote conversion."""

from collections import OrderedDict

from ...data_types import FeatureType


def get_feature_map():
    return OrderedDict([
        ("Original_Quote_Date", FeatureType.DATE),
        ("Field6", FeatureType.CATEGORICAL),
        ("Field7", FeatureType.CONTINUOUS),
        ("Field8", FeatureType.CONTINUOUS),
        ("Field9", FeatureType.CONTINUOUS),
        ("Field10", FeatureType.CONTINUOUS),
        ("Field11", FeatureType.CONTINUOUS),
        ("Field12", FeatureType.CATEGORICAL),
        ("CoverageField1A", FeatureType.CONTINUOUS),
        ("CoverageField1B", FeatureType.CONTINUOUS),
        ("CoverageField2A", FeatureType.CONTINUOUS),
        ("CoverageField2B", FeatureType.CONTINUOUS),
        ("CoverageField3A", FeatureType.CONTINUOUS),
        ("CoverageField3B", FeatureType.CONTINUOUS),
        ("CoverageField4A", FeatureType.CONTINUOUS),
        ("CoverageField4B", FeatureType.CONTINUOUS),
        ("CoverageField5A", FeatureType.CONTINUOUS),
        ("CoverageField5B", FeatureType.CONTINUOUS),
        ("CoverageField6A", FeatureType.CONTINUOUS),
        ("CoverageField6B", FeatureType.CONTINUOUS),
        ("CoverageField8", FeatureType.CATEGORICAL),
        ("CoverageField9", FeatureType.CATEGORICAL),
        ("CoverageField11A", FeatureType.CONTINUOUS),
        ("CoverageField11B", FeatureType.CONTINUOUS),
        ("SalesField1A", FeatureType.CONTINUOUS),
        ("SalesField1B", FeatureType.CONTINUOUS),
        ("SalesField2A", FeatureType.CONTINUOUS),
        ("SalesField2B", FeatureType.CONTINUOUS),
        ("SalesField3", FeatureType.CONTINUOUS),
        ("SalesField4", FeatureType.CONTINUOUS),
        ("SalesField5", FeatureType.CONTINUOUS),
        ("SalesField6", FeatureType.CONTINUOUS),
        ("SalesField7", FeatureType.CATEGORICAL),
        ("SalesField8", FeatureType.CONTINUOUS),
        ("SalesField9", FeatureType.CONTINUOUS),
        ("SalesField10", FeatureType.CONTINUOUS),
        ("SalesField11", FeatureType.CONTINUOUS),
        ("SalesField12", FeatureType.CONTINUOUS),
        ("SalesField13", FeatureType.CONTINUOUS),
        ("SalesField14", FeatureType.CONTINUOUS),
        ("SalesField15", FeatureType.CONTINUOUS),
        ("PersonalField1", FeatureType.CONTINUOUS),
        ("PersonalField2", FeatureType.CONTINUOUS),
        ("PersonalField4A", FeatureType.CONTINUOUS),
        ("PersonalField4B", FeatureType.CONTINUOUS),
        ("PersonalField5", FeatureType.CONTINUOUS),
        ("PersonalField6", FeatureType.CONTINUOUS),
        ("PersonalField7", FeatureType.CATEGORICAL),
        ("PersonalField8", FeatureType.CONTINUOUS),
        ("PersonalField9", FeatureType.CONTINUOUS),
        ("PersonalField10A", FeatureType.CONTINUOUS),
        ("PersonalField10B", FeatureType.CONTINUOUS),
        ("PersonalField11", FeatureType.CONTINUOUS),
        ("PersonalField12", FeatureType.CONTINUOUS),
        ("PersonalField13", FeatureType.CONTINUOUS),
        ("PersonalField14", FeatureType.CONTINUOUS),
        ("PersonalField15", FeatureType.CONTINUOUS),
        ("PersonalField16", FeatureType.CATEGORICAL),
        ("PersonalField17", FeatureType.CATEGORICAL),
        ("PersonalField18", FeatureType.CATEGORICAL),
        ("PersonalField19", FeatureType.CATEGORICAL),
        ("PersonalField22", FeatureType.CONTINUOUS),
        ("PersonalField23", FeatureType.CONTINUOUS),
        ("PersonalField24", FeatureType.CONTINUOUS),
        ("PersonalField25", FeatureType.CONTINUOUS),
        ("PersonalField26", FeatureType.CONTINUOUS),
        ("PersonalField27", FeatureType.CONTINUOUS),
        ("PersonalField28", FeatureType.CONTINUOUS),
        ("PersonalField29", FeatureType.CONTINUOUS),
        ("PersonalField30", FeatureType.CONTINUOUS),
        ("PersonalField31", FeatureType.CONTINUOUS),
        ("PersonalField32", FeatureType.CONTINUOUS),
        ("PersonalField33", FeatureType.CONTINUOUS),
        ("PersonalField34", FeatureType.CONTINUOUS),
        ("PersonalField35", FeatureType.CONTINUOUS),
        ("PersonalField36", FeatureType.CONTINUOUS),
        ("PersonalField37", FeatureType.CONTINUOUS),
        ("PersonalField38", FeatureType.CONTINUOUS),
        ("PersonalField39", FeatureType.CONTINUOUS),
        ("PersonalField40", FeatureType.CONTINUOUS),
        ("PersonalField41", FeatureType.CONTINUOUS),
        ("PersonalField42", FeatureType.CONTINUOUS),
        ("PersonalField43", FeatureType.CONTINUOUS),
        ("PersonalField44", FeatureType.CONTINUOUS),
        ("PersonalField45", FeatureType.CONTINUOUS),
        ("PersonalField46", FeatureType.CONTINUOUS),
        ("PersonalField47", FeatureType.CONTINUOUS),
        ("PersonalField48", FeatureType.CONTINUOUS),
        ("PersonalField49", FeatureType.CONTINUOUS),
        ("PersonalField50", FeatureType.CONTINUOUS),
        ("PersonalField51", FeatureType.CONTINUOUS),
        ("PersonalField52", FeatureType.CONTINUOUS),
        ("PersonalField53", FeatureType.CONTINUOUS),
        ("PersonalField54", FeatureType.CONTINUOUS),
        ("PersonalField55", FeatureType.CONTINUOUS),
        ("PersonalField56", FeatureType.CONTINUOUS),
        ("PersonalField57", FeatureType.CONTINUOUS),
        ("PersonalField58", FeatureType.CONTINUOUS),
        ("PersonalField59", FeatureType.CONTINUOUS),
        ("PersonalField60", FeatureType.CONTINUOUS),
        ("PersonalField61", FeatureType.CONTINUOUS),
        ("PersonalField62", FeatureType.CONTINUOUS),
        ("PersonalField63", FeatureType.CONTINUOUS),
        ("PersonalField64", FeatureType.CONTINUOUS),
        ("PersonalField65", FeatureType.CONTINUOUS),
        ("PersonalField66", FeatureType.CONTINUOUS),
        ("PersonalField67", FeatureType.CONTINUOUS),
        ("PersonalField68", FeatureType.CONTINUOUS),
        ("PersonalField69", FeatureType.CONTINUOUS),
        ("PersonalField70", FeatureType.CONTINUOUS),
        ("PersonalField71", FeatureType.CONTINUOUS),
        ("PersonalField72", FeatureType.CONTINUOUS),
        ("PersonalField73", FeatureType.CONTINUOUS),
        ("PersonalField74", FeatureType.CONTINUOUS),
        ("PersonalField75", FeatureType.CONTINUOUS),
        ("PersonalField76", FeatureType.CONTINUOUS),
        ("PersonalField77", FeatureType.CONTINUOUS),
        ("PersonalField78", FeatureType.CONTINUOUS),
        ("PersonalField79", FeatureType.CONTINUOUS),
        ("PersonalField80", FeatureType.CONTINUOUS),
        ("PersonalField81", FeatureType.CONTINUOUS),
        ("PersonalField82", FeatureType.CONTINUOUS),
        ("PersonalField83", FeatureType.CONTINUOUS),
        ("PersonalField84", FeatureType.CATEGORICAL),
        ("PropertyField1A", FeatureType.CONTINUOUS),
        ("PropertyField1B", FeatureType.CONTINUOUS),
        ("PropertyField2A", FeatureType.CONTINUOUS),
        ("PropertyField2B", FeatureType.CONTINUOUS),
        ("PropertyField3", FeatureType.CATEGORICAL),
        ("PropertyField4", FeatureType.CATEGORICAL),
        ("PropertyField5", FeatureType.CATEGORICAL),
        ("PropertyField6", FeatureType.CONTINUOUS),
        ("PropertyField7", FeatureType.CATEGORICAL),
        ("PropertyField8", FeatureType.CONTINUOUS),
        ("PropertyField9", FeatureType.CONTINUOUS),
        ("PropertyField10", FeatureType.CONTINUOUS),
        ("PropertyField11A", FeatureType.CONTINUOUS),
        ("PropertyField11B", FeatureType.CONTINUOUS),
        ("PropertyField12", FeatureType.CONTINUOUS),
        ("PropertyField13", FeatureType.CONTINUOUS),
        ("PropertyField14", FeatureType.CATEGORICAL),
        ("PropertyField15", FeatureType.CONTINUOUS),
        ("PropertyField16A", FeatureType.CONTINUOUS),
        ("PropertyField16B", FeatureType.CONTINUOUS),
        ("PropertyField17", FeatureType.CONTINUOUS),
        ("PropertyField18", FeatureType.CONTINUOUS),
        ("PropertyField19", FeatureType.CONTINUOUS),
        ("PropertyField20", FeatureType.CONTINUOUS),
        ("PropertyField21A", FeatureType.CONTINUOUS),
        ("PropertyField21B", FeatureType.CONTINUOUS),
        ("PropertyField22", FeatureType.CONTINUOUS),
        ("PropertyField23", FeatureType.CONTINUOUS),
        ("PropertyField24A", FeatureType.CONTINUOUS),
        ("PropertyField24B", FeatureType.CONTINUOUS),
        ("PropertyField25", FeatureType.CONTINUOUS),
        ("PropertyField26A", FeatureType.CONTINUOUS),
        ("PropertyField26B", FeatureType.CONTINUOUS),
        ("PropertyField27", FeatureType.CONTINUOUS),
        ("PropertyField28", FeatureType.CATEGORICAL),
        ("PropertyField29", FeatureType.CATEGORICAL),
        ("PropertyField30", FeatureType.CATEGORICAL),
        ("PropertyField31", FeatureType.CATEGORICAL),
        ("PropertyField32", FeatureType.CATEGORICAL),
        ("PropertyField33", FeatureType.CATEGORICAL),
        ("PropertyField34", FeatureType.CATEGORICAL),
        ("PropertyField35", FeatureType.CONTINUOUS),
        ("PropertyField36", FeatureType.CATEGORICAL),
        ("PropertyField37", FeatureType.CATEGORICAL),
        ("PropertyField38", FeatureType.CATEGORICAL),
        ("PropertyField39A", FeatureType.CONTINUOUS),
        ("PropertyField39B", FeatureType.CONTINUOUS),
        ("GeographicField1A", FeatureType.CONTINUOUS),
        ("GeographicField1B", FeatureType.CONTINUOUS),
        ("GeographicField2A", FeatureType.CONTINUOUS),
        ("GeographicField2B", FeatureType.CONTINUOUS),
        ("GeographicField3A", FeatureType.CONTINUOUS),
        ("GeographicField3B", FeatureType.CONTINUOUS),
        ("GeographicField4A", FeatureType.CONTINUOUS),
        ("GeographicField4B", FeatureType.CONTINUOUS),
        ("GeographicField5A", FeatureType.CONTINUOUS),
        ("GeographicField5B", FeatureType.CONTINUOUS),
        ("GeographicField6A", FeatureType.CONTINUOUS),
        ("GeographicField6B", FeatureType.CONTINUOUS),
        ("GeographicField7A", FeatureType.CONTINUOUS),
        ("GeographicField7B", FeatureType.CONTINUOUS),
        ("GeographicField8A", FeatureType.CONTINUOUS),
        ("GeographicField8B", FeatureType.CONTINUOUS),
        ("GeographicField9A", FeatureType.CONTINUOUS),
        ("GeographicField9B", FeatureType.CONTINUOUS),
        ("GeographicField10A", FeatureType.CONTINUOUS),
        ("GeographicField10B", FeatureType.CONTINUOUS),
        ("GeographicField11A", FeatureType.CONTINUOUS),
        ("GeographicField11B", FeatureType.CONTINUOUS),
        ("GeographicField12A", FeatureType.CONTINUOUS),
        ("GeographicField12B", FeatureType.CONTINUOUS),
        ("GeographicField13A", FeatureType.CONTINUOUS),
        ("GeographicField13B", FeatureType.CONTINUOUS),
        ("GeographicField14A", FeatureType.CONTINUOUS),
        ("GeographicField14B", FeatureType.CONTINUOUS),
        ("GeographicField15A", FeatureType.CONTINUOUS),
        ("GeographicField15B", FeatureType.CONTINUOUS),
        ("GeographicField16A", FeatureType.CONTINUOUS),
        ("GeographicField16B", FeatureType.CONTINUOUS),
        ("GeographicField17A", FeatureType.CONTINUOUS),
        ("GeographicField17B", FeatureType.CONTINUOUS),
        ("GeographicField18A", FeatureType.CONTINUOUS),
        ("GeographicField18B", FeatureType.CONTINUOUS),
        ("GeographicField19A", FeatureType.CONTINUOUS),
        ("GeographicField19B", FeatureType.CONTINUOUS),
        ("GeographicField20A", FeatureType.CONTINUOUS),
        ("GeographicField20B", FeatureType.CONTINUOUS),
        ("GeographicField21A", FeatureType.CONTINUOUS),
        ("GeographicField21B", FeatureType.CONTINUOUS),
        ("GeographicField22A", FeatureType.CONTINUOUS),
        ("GeographicField22B", FeatureType.CONTINUOUS),
        ("GeographicField23A", FeatureType.CONTINUOUS),
        ("GeographicField23B", FeatureType.CONTINUOUS),
        ("GeographicField24A", FeatureType.CONTINUOUS),
        ("GeographicField24B", FeatureType.CONTINUOUS),
        ("GeographicField25A", FeatureType.CONTINUOUS),
        ("GeographicField25B", FeatureType.CONTINUOUS),
        ("GeographicField26A", FeatureType.CONTINUOUS),
        ("GeographicField26B", FeatureType.CONTINUOUS),
        ("GeographicField27A", FeatureType.CONTINUOUS),
        ("GeographicField27B", FeatureType.CONTINUOUS),
        ("GeographicField28A", FeatureType.CONTINUOUS),
        ("GeographicField28B", FeatureType.CONTINUOUS),
        ("GeographicField29A", FeatureType.CONTINUOUS),
        ("GeographicField29B", FeatureType.CONTINUOUS),
        ("GeographicField30A", FeatureType.CONTINUOUS),
        ("GeographicField30B", FeatureType.CONTINUOUS),
        ("GeographicField31A", FeatureType.CONTINUOUS),
        ("GeographicField31B", FeatureType.CONTINUOUS),
        ("GeographicField32A", FeatureType.CONTINUOUS),
        ("GeographicField32B", FeatureType.CONTINUOUS),
        ("GeographicField33A", FeatureType.CONTINUOUS),
        ("GeographicField33B", FeatureType.CONTINUOUS),
        ("GeographicField34A", FeatureType.CONTINUOUS),
        ("GeographicField34B", FeatureType.CONTINUOUS),
        ("GeographicField35A", FeatureType.CONTINUOUS),
        ("GeographicField35B", FeatureType.CONTINUOUS),
        ("GeographicField36A", FeatureType.CONTINUOUS),
        ("GeographicField36B", FeatureType.CONTINUOUS),
        ("GeographicField37A", FeatureType.CONTINUOUS),
        ("GeographicField37B", FeatureType.CONTINUOUS),
        ("GeographicField38A", FeatureType.CONTINUOUS),
        ("GeographicField38B", FeatureType.CONTINUOUS),
        ("GeographicField39A", FeatureType.CONTINUOUS),
        ("GeographicField39B", FeatureType.CONTINUOUS),
        ("GeographicField40A", FeatureType.CONTINUOUS),
        ("GeographicField40B", FeatureType.CONTINUOUS),
        ("GeographicField41A", FeatureType.CONTINUOUS),
        ("GeographicField41B", FeatureType.CONTINUOUS),
        ("GeographicField42A", FeatureType.CONTINUOUS),
        ("GeographicField42B", FeatureType.CONTINUOUS),
        ("GeographicField43A", FeatureType.CONTINUOUS),
        ("GeographicField43B", FeatureType.CONTINUOUS),
        ("GeographicField44A", FeatureType.CONTINUOUS),
        ("GeographicField44B", FeatureType.CONTINUOUS),
        ("GeographicField45A", FeatureType.CONTINUOUS),
        ("GeographicField45B", FeatureType.CONTINUOUS),
        ("GeographicField46A", FeatureType.CONTINUOUS),
        ("GeographicField46B", FeatureType.CONTINUOUS),
        ("GeographicField47A", FeatureType.CONTINUOUS),
        ("GeographicField47B", FeatureType.CONTINUOUS),
        ("GeographicField48A", FeatureType.CONTINUOUS),
        ("GeographicField48B", FeatureType.CONTINUOUS),
        ("GeographicField49A", FeatureType.CONTINUOUS),
        ("GeographicField49B", FeatureType.CONTINUOUS),
        ("GeographicField50A", FeatureType.CONTINUOUS),
        ("GeographicField50B", FeatureType.CONTINUOUS),
        ("GeographicField51A", FeatureType.CONTINUOUS),
        ("GeographicField51B", FeatureType.CONTINUOUS),
        ("GeographicField52A", FeatureType.CONTINUOUS),
        ("GeographicField52B", FeatureType.CONTINUOUS),
        ("GeographicField53A", FeatureType.CONTINUOUS),
        ("GeographicField53B", FeatureType.CONTINUOUS),
        ("GeographicField54A", FeatureType.CONTINUOUS),
        ("GeographicField54B", FeatureType.CONTINUOUS),
        ("GeographicField55A", FeatureType.CONTINUOUS),
        ("GeographicField55B", FeatureType.CONTINUOUS),
        ("GeographicField56A", FeatureType.CONTINUOUS),
        ("GeographicField56B", FeatureType.CONTINUOUS),
        ("GeographicField57A", FeatureType.CONTINUOUS),
        ("GeographicField57B", FeatureType.CONTINUOUS),
        ("GeographicField58A", FeatureType.CONTINUOUS),
        ("GeographicField58B", FeatureType.CONTINUOUS),
        ("GeographicField59A", FeatureType.CONTINUOUS),
        ("GeographicField59B", FeatureType.CONTINUOUS),
        ("GeographicField60A", FeatureType.CONTINUOUS),
        ("GeographicField60B", FeatureType.CONTINUOUS),
        ("GeographicField61A", FeatureType.CONTINUOUS),
        ("GeographicField61B", FeatureType.CONTINUOUS),
        ("GeographicField62A", FeatureType.CONTINUOUS),
        ("GeographicField62B", FeatureType.CONTINUOUS),
        ("GeographicField63", FeatureType.CATEGORICAL),
        ("GeographicField64", FeatureType.CATEGORICAL),
    ])
