# TODO: these should be enum
ONE_HOT_ENCODER = "one_hot_encoder"
IMPUTER = "imputer"
RESCALER = "rescaler"
FEATURE_PREPROCESSOR = "feature_preprocessor"
CLASSIFIER = "classifier"
REGRESSOR = "regressor"
ALGORITHM_TYPES = [
    ONE_HOT_ENCODER,
    IMPUTER,
    RESCALER,
    FEATURE_PREPROCESSOR,
    CLASSIFIER,
    REGRESSOR,
]
START_TOKEN = "<sos>"
END_TOKEN = "<eos>"
NONE_TOKEN = "<none>"
